#!/usr/bin/env perl

use strict;
use warnings;

use Cwd;
use File::Basename 'dirname';
use VDTGlue;

my %defs = load_defs();
my $dir = cwd;

# Determine OpenSSL root directory for later use in commands
my $OPENSSL_ROOT = dirname(dirname(`which openssl`));

# Don't continue if a previous step failed
die "Previous step failed, exiting\n" if $ENV{_NMI_STEP_FAILED};

# Dispatch to the correct task
die "Not a valid task: $ENV{_NMI_TASKNAME}\n" unless $main::{$ENV{_NMI_TASKNAME}};
$main::{$ENV{_NMI_TASKNAME}}();

sub configure_apache {
    chdir "$dir/httpd-$defs{APACHE_VERSION}";

    # Apply our patches
    system("patch -p0 < $dir/apachectl.patch");
    system("patch -p0 < $dir/GSI.patch");

    # Base configuration
    my $configure_flags = "";
    if ($ENV{NMI_PLATFORM} =~ /rhas_3/) {
        # With Apache 2.2.11, they changed how they call pkg-config, and they don't 
        # know about this flag. So we force it. 
        if ($ENV{NMI_PLATFORM} =~ /x86_64/) {
            $configure_flags = "LDFLAGS=\"-L/usr/kerberos/lib -L/usr/kerberos/lib64\"";
        } else {
            $configure_flags = "LDFLAGS=-L/usr/kerberos/lib";
        }
    }
    my $configure_command = ("$configure_flags ./configure --prefix=$dir/apache " .
                             "--with-perl='/usr/bin/env perl' " .
                             '--enable-proxy=shared ' .
                             '--enable-proxy-httpd=shared ' .
                             '--enable-rewrite=shared ' .
                             '--enable-expires=shared ' .
                             '--enable-headers=shared ' .
                             '--without-gdbm ' .
                             '--without-ndbm ' .
                             '--without-sqlite3 ' .
                             '--without-sqlite2 ' .
                             '--without-berkeley-db ' .
                             '--enable-ssl ' .
                             "--with-ssl=$OPENSSL_ROOT");

    # Use IBM compiler
    if ($ENV{NMI_PLATFORM} =~ /_aix_/) {
        $configure_command .= " CC=xlc_r CXX=xlC_r";
    }

    # Run configure
    system($configure_command);

    # For some reason on x86_64_rhap_5, our build fails because configure
    # decides that we need to put /usr/lib in our library search path ahead of /lib64.
    # It looks like this:
    #    checking for SSL_set_cert_store... no
    #      adding "-I/usr/include" to INCLUDES
    #      adding "-I/usr/kerberos/include" to INCLUDES
    #      adding "-L/usr/lib" to LDFLAGS
    #      forcing SSL_LIBS to "-lssl -lcrypto"
    #      adding "-L/usr/kerberos/lib64" to SSL_LIBS
    #      adding "-ldl" to SSL_LIBS
    #      adding "-lz" to SSL_LIBS
    #      forcing MOD_SSL_LDADD to "$(SSL_LIBS)"
    # That's all fine and dandy until we try to link against expat, when we get:
    #      /usr/lib/libexpat.so: could not read symbols: File in wrong format
    # So on this platform, I have out EXTRA_LDFLAGS. The build now works. I'm not convinced
    # tht this is the best fix, but I couldn't figure out why it wants to add this. 
    if ($ENV{NMI_PLATFORM} =~ /(x86_64_rhas_4|x86_64_rhap_5)/) {
        system("perl -pi -e 's/^EXTRA_LDFLAGS = .*/EXTRA_LDFLAGS = /' build/config_vars.mk")
    }

}

sub build_apache {
    chdir "$dir/httpd-$defs{APACHE_VERSION}";

    system("make");
    system("make install");

    # Save off the libuuid since it's not always installed
    foreach my $lib (qw(/lib/libuuid.so.1 /lib64/libuuid.so.1)) {
	next unless -e $lib;
	system("cp -f -d $lib* $dir/apache/lib");
    }
}
 
sub mod_jk {
    # Build and install mod_jk
    chdir "tomcat-connectors-$defs{MODJK_VERSION}-src/native"
        or die "Could not cd tomcat-connectors-$defs{MODJK_VERSION}-src/native: $!";
    system("ls -l");

    # Set up and run configure
    my $configure_command = "./configure --with-apxs=$dir/apache/bin/apxs --prefix=$dir/apache";
    if ($ENV{NMI_PLATFORM} =~ /_aix_/) {
        $configure_command .= " CC=xlc_r CXX=xlC_r LIBPATH=$OPENSSL_ROOT/lib";
    }
    system($configure_command);

    my $make_command = 'make';
    if ($ENV{NMI_PLATFORM} =~ /_aix_/) {
        $make_command .= " LIBPATH=$OPENSSL_ROOT/lib";
    }
    system($make_command);

    my $make_install_command = 'make install';
    if ($ENV{NMI_PLATFORM} =~ /_aix_/) {
        $make_install_command .= " LIBPATH=$OPENSSL_ROOT/lib";
    }
    system($make_install_command);
}

sub libxml {
    chdir "libxml2-$defs{LIBXML_VERSION}";

    # Set up and run configure
    my $configure_command = "./configure --prefix=$dir/apache";
    if ($ENV{NMI_PLATFORM} =~ /_aix_/) {
        $configure_command .= " CC=xlc_r CXX=xlC_r";
    }
    system($configure_command);

    system("make");
    system("make install");
}

sub bundle {
    # Tar it all up
    system("tar cvfz apache-$defs{APACHE_VERSION}.$ENV{NMI_PLATFORM}.tar.gz apache");
    system("tar cvfz results.tar.gz apache-$defs{APACHE_VERSION}.$ENV{NMI_PLATFORM}.tar.gz");
}
